@@ -81,6 +81,7 @@ static void 		exchange_subscribe_folder (CamelStore *store,
 static void 		exchange_unsubscribe_folder (CamelStore *store,
 						const char *folder_name,
 						CamelException *ex);
+static gboolean exchange_can_refresh_folder (CamelStore *store, CamelFolderInfo *info, CamelException *ex);
 
 static void stub_notification (CamelObject *object, gpointer event_data, gpointer user_data);
 
@@ -112,6 +113,7 @@ class_init (CamelExchangeStoreClass *camel_exchange_store_class)
 	camel_store_class->folder_subscribed = exchange_folder_subscribed;
 	camel_store_class->subscribe_folder = exchange_subscribe_folder;
 	camel_store_class->unsubscribe_folder = exchange_unsubscribe_folder;
+	camel_store_class->can_refresh_folder = exchange_can_refresh_folder;
 }
 
 static void
@@ -345,17 +347,17 @@ static void
 camel_exchange_get_password (CamelService *service, CamelException *ex)
 {
 	CamelSession *session = camel_service_get_session (service);
-	char *prompt, *account_name;
 
 	if (!service->url->passwd) {
-		/* This is a hack to use the same string being used in
-		   exchange-account.c:get_password */
-		account_name = g_strdup_printf ("%s@%s", service->url->user, service->url->host);
-		prompt = g_strdup_printf (_("%sEnter password for %s"), "", account_name);
+		gchar *prompt;
+
+		prompt = camel_session_build_password_prompt (
+			"Exchange", service->url->user, service->url->host);
+
+		service->url->passwd = camel_session_get_password (
+			session, service, "Exchange", prompt,
+			"password", CAMEL_SESSION_PASSWORD_SECRET, ex);
 
-		service->url->passwd = camel_session_get_password (session,
-					service, "Exchange", prompt, "password", CAMEL_SESSION_PASSWORD_SECRET, ex);
-		g_free (account_name);
 		g_free (prompt);
 	}
 }
@@ -1142,3 +1144,22 @@ stub_notification (CamelObject *object, gpointer event_data, gpointer user_data)
 		break;
 	}
 }
+
+static gboolean
+exchange_can_refresh_folder (CamelStore *store, CamelFolderInfo *info, CamelException *ex)
+{
+	gboolean res;
+
+	res = CAMEL_STORE_CLASS(parent_class)->can_refresh_folder (store, info, ex) ||
+	      (camel_url_get_param (((CamelService *)store)->url, "check_all") != NULL);
+
+	if (!res && !camel_exception_is_set (ex)) {
+		CamelFolder *folder;
+
+		folder = camel_store_get_folder (store, info->full_name, 0, ex);
+		if (folder && CAMEL_IS_EXCHANGE_FOLDER (folder))
+			res = CAMEL_EXCHANGE_FOLDER (folder)->check_folder;
+	}
+
+	return res;
+}
